/* GNOME DB library
 * Copyright (C) 1999 Rodrigo Moya
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#if defined(HAVE_CONFIG_H)
#  include <config.h>
#endif
#include <gnome-db.h>

static void gnome_db_combo_class_init (GnomeDbComboClass *klass);
static void gnome_db_combo_init       (GnomeDbCombo *dbcombo);

/*
 * GnomeDbCombo widget signals
 */
enum
{
  GNOME_DB_COMBO_ERROR,
  GNOME_DB_COMBO_SELECTION_CHANGED,
  GNOME_DB_COMBO_LAST_SIGNAL
};

static gint gnome_db_combo_signals[GNOME_DB_COMBO_LAST_SIGNAL] = { 0, };

/*
 * Private functions
 */
static void
show_recordset (GnomeDbCombo *dbcombo, gint pos)
{
  GList *strings = NULL;
  gtk_list_clear_items(GTK_LIST(GTK_COMBO(dbcombo)->list), 0, -1);
  if (dbcombo->recset != NULL)
    {
      gulong position = gda_recordset_move(dbcombo->recset, 1, 0);
      while (position != GDA_RECORDSET_INVALID_POSITION &&
             !gda_recordset_eof(dbcombo->recset))
        {
          Gda_Field *field = gda_recordset_field_idx(dbcombo->recset, pos);
          strings = g_list_append(strings, gda_stringify_value(0, 0, field));
          /* fetch next row */
          position = gda_recordset_move(dbcombo->recset, 1, 0);
        }
      gtk_combo_set_popdown_strings(GTK_COMBO(dbcombo), strings);
    }
}

/*
 * Callbacks
 */
static void
selection_changed_cb (GtkWidget *w, gpointer data)
{
  GnomeDbCombo *dbcombo;
  gchar *str;
  
  g_return_if_fail(GTK_IS_LIST(w));
  g_return_if_fail(GNOME_DB_IS_COMBO(data));
  
  dbcombo = GNOME_DB_COMBO(data);
  
  /* retrieve GtkCombo's current selection */
  str = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dbcombo)->entry));
  if (str != 0)
    {
      /* emit GnomeDbCombo's "selection_changed" signal */
      gtk_signal_emit_by_name(GTK_OBJECT(dbcombo), "selection_changed", str);
    }
}

/*
 * GnomeDbCombo widget interface
 */
static void
gnome_db_combo_class_init (GnomeDbComboClass *klass)
{
  GtkObjectClass *object_class = (GtkObjectClass *) klass;

  gnome_db_combo_signals[GNOME_DB_COMBO_ERROR] =
          gtk_signal_new ("error", GTK_RUN_FIRST, object_class->type,
                          GTK_SIGNAL_OFFSET(GnomeDbComboClass, error),
                          gtk_signal_default_marshaller,
                          GTK_TYPE_NONE, 0);
  gnome_db_combo_signals[GNOME_DB_COMBO_SELECTION_CHANGED] =
          gtk_signal_new("selection_changed", GTK_RUN_FIRST, object_class->type,
                         GTK_SIGNAL_OFFSET(GnomeDbComboClass, selection_changed),
                         gtk_signal_default_marshaller,
                         GTK_TYPE_NONE, 0);
  gtk_object_class_add_signals(object_class, gnome_db_combo_signals,
                               GNOME_DB_COMBO_LAST_SIGNAL);

  klass->error = 0;
  klass->selection_changed = 0;
  object_class->destroy = gnome_db_combo_destroy;
}

static void
gnome_db_combo_init (GnomeDbCombo *dbcombo)
{
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dbcombo)->entry), FALSE);
  dbcombo->recset = 0;
  dbcombo->total_rows = 0;
  dbcombo->editable = FALSE;
  
  /* connect to GtkList's "selection_changed" signal */
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(dbcombo)->list), "selection_changed",
                     GTK_SIGNAL_FUNC(selection_changed_cb), (gpointer) dbcombo);
}

guint
gnome_db_combo_get_type (void)
{
  static guint db_combo_type = 0;
  
  if (!db_combo_type)
    {
      GtkTypeInfo db_combo_info =
      {
        "GnomeDbCombo",
        sizeof (GnomeDbCombo),
        sizeof (GnomeDbComboClass),
        (GtkClassInitFunc) gnome_db_combo_class_init,
        (GtkObjectInitFunc) gnome_db_combo_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };
      db_combo_type = gtk_type_unique(gtk_combo_get_type(), &db_combo_info);
    }
  return (db_combo_type);
}

/**
 * gnome_db_combo_new
 * @recset: recordset to be shown
 * @pos: position of the field being shown
 *
 * Creates a new GnomeDbCombo widget. You can optionally specify a Gda_Recordset
 * object which will be used by the GnomeDbCombo widget
 *
 * Returns: a pointer to the new widget, or NULL on error
 */
GtkWidget *
gnome_db_combo_new (Gda_Recordset *recset, gint pos)
{
  GtkWidget *dbcombo = gtk_type_new(gnome_db_combo_get_type());
  if (recset != 0)
    gnome_db_combo_set_recordset(GNOME_DB_COMBO(dbcombo), recset, pos);
  return (dbcombo);
}

void 
gnome_db_combo_destroy (GnomeDbCombo *dbcombo)
{
  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));

  if (dbcombo->recset != 0)
    {
      gda_recordset_free(dbcombo->recset);
    }
}

/**
 * gnome_db_combo_get_editable
 * @dbcombo: the GnomeDbCombo widget
 * Returns: TRUE if the combo box is editable , or FALSE if not
 */
gboolean
gnome_db_combo_get_editable (GnomeDbCombo *dbcombo)
{
  g_return_val_if_fail(GNOME_DB_IS_COMBO(dbcombo), FALSE);
  return (dbcombo->editable);
}

void
gnome_db_combo_set_editable (GnomeDbCombo *dbcombo, gboolean editable)
{
  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dbcombo)->entry), editable);
  dbcombo->editable = editable;
}

void
gnome_db_combo_refresh (GnomeDbCombo *dbcombo)
{
  g_return_if_fail(dbcombo != NULL);
  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));
}

gint
gnome_db_combo_get_row_count (GnomeDbCombo *dbcombo)
{
  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));
  return (dbcombo->total_rows);
}

gchar *
gnome_db_combo_get_string (GnomeDbCombo *dbcombo)
{
  g_return_val_if_fail(dbcombo != NULL, NULL);
  g_return_val_if_fail(GNOME_DB_IS_COMBO(dbcombo), NULL);
  return (gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dbcombo)->entry)));
}

Gda_Recordset *
gnome_db_combo_get_recordset (GnomeDbCombo *dbcombo)
{
  g_return_val_if_fail(dbcombo != NULL, NULL);
  g_return_val_if_fail(GNOME_DB_IS_COMBO(dbcombo), NULL);
  return (dbcombo->recset);
}

void
gnome_db_combo_set_recordset (GnomeDbCombo *dbcombo, Gda_Recordset *recset, gint pos)
{
  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));
  g_return_if_fail(IS_GDA_RECORDSET(recset));
  g_return_if_fail(pos >= 0);
  
  if (recset != NULL && recset == dbcombo->recset)
    gnome_db_combo_refresh(dbcombo);
  else
    {
      if (dbcombo->recset != 0)
        {
          gda_recordset_close(dbcombo->recset);
          gda_recordset_free(dbcombo->recset);
        }
      dbcombo->recset = recset;
      show_recordset(dbcombo, pos);
    }
}

void
gnome_db_combo_sync (GnomeDbCombo *dbcombo)
{
  GList*     selected;
  GtkWidget* item;
  gint       row_index;

  g_return_if_fail(GNOME_DB_IS_COMBO(dbcombo));
  g_return_if_fail(IS_GDA_RECORDSET(dbcombo->recset));

  /* find the index of the current combo selection */
  selected = GTK_LIST(GTK_COMBO(dbcombo)->list)->selection;
  if (selected)
    {
      item = GTK_WIDGET(selected->data);
      row_index = gtk_list_child_position(GTK_LIST(GTK_COMBO(dbcombo)->list), 
                                          item);

      /* reposition the record set */
      gda_recordset_move_first(dbcombo->recset);
      gda_recordset_move(dbcombo->recset, row_index, NULL);
    }
}
